<div align="center">
    <h2 align="center">
        <img src="https://forge.kworld.fr/foss/nabu/-/raw/master/nabu_logo.png" width="120"/>
<br>Nabû: <small>Standardizing .NET developments with abstractions</small>
    </h2>
    <p align="center">
     <a href="https://dotnet.microsoft.com/">
        <img src="https://img.shields.io/badge/⩾-6.0-512BD4.svg?stylee&logo=dotnet" alt=".Net" />
        </a>     
<a href="https://dotnet.microsoft.com/en-us/languages/csharp">
    <img src="https://img.shields.io/badge/C%23-05930c.svg?stylee&logo=csharp" alt="C#" />
</a>

 <a href="https://forge.kworld.fr/foss/nabu/-/blob/master/LICENSE.txt">
    <img src="https://img.shields.io/badge/License-Apache%202.0-blue.svg" alt="apache2-license" />
    </a>    

<a href="https://forge.kworld.fr/foss/nabu/-/pipelines/latest?ref=master">
  <img src="https://img.shields.io/gitlab/pipeline-status/foss%2Fnabu?gitlab_url=https%3A%2F%2Fforge.kworld.fr%2F&branch=master&logo=gitlab" alt="CI Status" />
</a>    
</p>
<hr>
</div>


Nabû is an open-source suite of libraries designed to organize and standardize .NET applications development through streamlined interactions.
The suite is divided into three (for now) components:

- **Nabu.Core**: Provides inversion of control (IoC) helpers that streamline services registration by using attribute tagging.
- **Nabu.Data**: Provides data manipulation abstraction, encompassing CRUD operations and sophisticated querying using LinQ.
- **Nabu.Storage**: Provides standardized file manipulation abstractions, catering to local, remote, and cloud-based files.

#### Latest packages

<a href="https://nuget.kworld.dev/packages/nabu.core">
 <img src="https://img.shields.io/badge/dynamic/json?url=https%3A%2F%2Fnuget.kworld.dev%2Fv3%2Fregistration%2Fnabu.core%2Findex.json&query=%24.items%5B0%5D.upper&logo=nuget&label=Nabu.Core&color=purple" alt="Core Nuget" />
</a>
<br>
<a href="https://nuget.kworld.dev/packages/nabu.data">
 <img src="https://img.shields.io/badge/dynamic/json?url=https%3A%2F%2Fnuget.kworld.dev%2Fv3%2Fregistration%2Fnabu.data%2Findex.json&query=%24.items%5B0%5D.upper&logo=nuget&label=Nabu.Data&color=green" alt="Data Nuget" />
</a><a href="https://nuget.kworld.dev/packages/nabu.data.entityframework">
 <img src="https://img.shields.io/badge/dynamic/json?url=https%3A%2F%2Fnuget.kworld.dev%2Fv3%2Fregistration%2Fnabu.data.entityframework%2Findex.json&query=%24.items%5B0%5D.upper&logo=nuget&label=Nabu.Data.EntityFramework&color=darkgreen" alt="Data.EntityFramework Nuget" />
</a><a href="https://nuget.kworld.dev/packages/nabu.core">
 <img src="https://img.shields.io/badge/dynamic/json?url=https%3A%2F%2Fnuget.kworld.dev%2Fv3%2Fregistration%2Fnabu.data.litedb%2Findex.json&query=%24.items%5B0%5D.upper&logo=nuget&label=Nabu.Data.LiteDb&color=darkgreen" alt="Data.LiteDb Nuget" />
</a>
<br>
<a href="https://nuget.kworld.dev/packages/nabu.storage">
 <img src="https://img.shields.io/badge/dynamic/json?url=https%3A%2F%2Fnuget.kworld.dev%2Fv3%2Fregistration%2Fnabu.storage%2Findex.json&query=%24.items%5B0%5D.upper&logo=nuget&label=Nabu.Storage&color=blue" alt="Storage Nuget" />
</a><a href="https://nuget.kworld.dev/packages/nabu.storage.local">
 <img src="https://img.shields.io/badge/dynamic/json?url=https%3A%2F%2Fnuget.kworld.dev%2Fv3%2Fregistration%2Fnabu.storage.local%2Findex.json&query=%24.items%5B0%5D.upper&logo=nuget&label=Nabu.Storage.Local&color=darkblue" alt="Storage.Local Nuget" />
</a><a href="https://nuget.kworld.dev/packages/storage.minio">
 <img src="https://img.shields.io/badge/dynamic/json?url=https%3A%2F%2Fnuget.kworld.dev%2Fv3%2Fregistration%2Fnabu.storage.minio%2Findex.json&query=%24.items%5B0%5D.upper&logo=nuget&label=Nabu.Storage.Minio&color=darkblue" alt="Storage.Minio Nuget" />
</a>

---

## Installation

Before installing Nabû, make sure to add the custom NuGet source to your configuration.  
Open a terminal or command prompt and run the following command:

```bash
dotnet nuget add source "https://nuget.kworld.dev/v3/index.json" -n "kworld NuGet"
```

Now you can install the various components of Nabû with the following commands:

```bash
# Nabu.Core
dotnet add package my_awesome_project.csproj Nabu.Core

# Nabu.Data
dotnet add package my_awesome_project.csproj Nabu.Core --version 1.0.0-alpha
dotnet add package my_awesome_project.csproj Nabu.Data.EntityFramework --version 1.0.0-alpha # For EntityFramework implementation
dotnet add package my_awesome_project.csprojNabu.Data.LiteDb --version 1.0.0-alpha# For LiteDB implementation

# Nabu.Storage
dotnet add package my_awesome_project.csproj Nabu.Storage --version 1.0.0-alpha
dotnet add package my_awesome_project.csproj Nabu.Storage.Local --version 1.0.0-alpha # For local storage
dotnet add package my_awesome_project.csproj Nabu.Storage.Minio --version 1.0.0-alpha # For Minio (Amazon S3) storage
```

## Getting started

### Automatic Service Discovery

Nabu.Core introduces automatic service discovery. By tagging interfaces and classes with attributes, the ServiceHelper dynamically finds and uses implementations for services.

#### Usage

```csharp

// Example of using attribute ServiceAttribute for service registration
[Service]
public interface IMyService
{
    void DoSomething();
}

public class MyService : IMyService
{
    public void DoSomething()
    {
        // Implementation details
    }
}

public class Main
{
    public void ConfigureServices()
    {
        ServiceHelper serviceHelper = new();
        
        //by using serviceHelper.Context it's possible to precise
        //in wich assemblies or type collections search for implementations and definitions
        //by default it will search in AppDomain.GetAssemblies() (beware lazy-loading !)
        Assembly[] servicesImplementationsAssemblies =
        {
            typeof(MyService).Assembly,
        };
        serviceHelper.Context.WhiteListedDefinitionsTypes.Add(typeof(IMyService));
        serviceHelper.Context.ImplementationsAssemblies.AddRange(servicesImplementationsAssemblies); 
        
        
        //Launch the search of service definitions and theirs implement and then register them.
        serviceHelper.RegisterServices();
        
        using IServiceRegisterProvider provider = ServiceRegistry.BuildServiceProvider();
        
       
        IMyService? service = provider.GetService<IMyService>();
        service.DoSomething();
    }
}
```

### IDao: Data manipulation Interface

Nabu.Data provide the IDao interface in order to standardize data CRUD.

#### Usage

```csharp

public class User : IEntity
{
    public string Id { get; set; } = Guid.NewGuid().ToString();
    public DateTime Creation { get; set; } = DateTime.UtcNow;
    public DateTime Update { get; set; } = DateTime.UtcNow;
    
    public string? Name { get; set; }
    public int Age { get; set; }
    
    public List<Car> Cars { get; set; } = new();
}


public class Car : IEntity
{
    public string Id { get; set; } = Guid.NewGuid().ToString();
    public DateTime Creation { get; set; } = DateTime.UtcNow;
    public DateTime Update { get; set; } = DateTime.UtcNow;

    public string? Imatriculation { get; set; }
    public string? Model { get; set; }
    
    [ForeignKey("OwnerId")]
    public User? Owner {get;set;}
    public string? OwnerId { get; set; }
}

public class Main
{
    public async Task ExempleMethod()
    {
        ServiceHelper serviceHelper = new();
        //Configuration
        //...
        await using IServiceRegisterProvider provider = ServiceRegistry.BuildServiceProvider();
        await using IServiceRegistryScope scope = provider.CreateAsyncScope();
        await using IServiceRegisterProvider scopeProvider = scope.ServiceProvider;
        
        IDao? dao = scopeProvider.GetService<IDao>();
        
        
        User cosette = new() { Name = "Cosette", Age = 25 };
        User jean = new() { Name = "Jean Valjean", Age = 55 };
        
        cosette = await dao.AddAsync(cosette);
        jean = await dao.AddAsync(jean);
        
        Car car = new() { Imatriculation = $"CCC-BBB-1", OwnerId = null};
        await dao.AddAsync(car);
        
        Car car2 = new() { Imatriculation = $"CCC-BBB-2", OwnerId = cosette.Id};
        await dao.AddAsync(car2);
        
        car.OwnerId = cosette.id;
        await dao.UpdateAsync(car);
        
        ISpecification<Car> carSpecification = new Specification<Car> { Criteria = c => c.OwnerId == cosette.Id };
        carSpecification.OrderBy(c => c.Creation).ThenBy(c => c.Update);
        
        var dbCars = await dao.ListAllAsync(carSpecification);
        
    }
}


```

## Documentation

For comprehensive insights into the capabilities and nuances of Nabû, refer to the official documentation:

[Documentation Link]()

## Contributing

Join us in enhancing Nabû! We welcome contributions in the form of suggestions, bug reports, and pull requests. Check out the [Contribution Guidelines](https://forge.kworld.fr/foss/nabu/-/blob/master/CONTRIBUTING.md) to get involved.

## License

This project operates under the Apache 2.0 License. For more details, refer to the [LICENSE](https://forge.kworld.fr/foss/nabu/-/blob/master/LICENSE.txt) file.

## Why "Nabû" ?

__Nabû__ (Akkadian: cuneiform: 𒀭𒀝) is the name of and old Mesopotamian God.  
In ancient cultures, __Nabû__ was revered as a deity associated with wisdom, scholarship, and the recording of informations. __Nabû__'s symbols included a stylus resting on a tablet.

By adopting those name and symbols, this suite of libraries refers, in a cool way, to the the art of writing and recording of informations.
